@@ -14,7 +14,6 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
-
 //! MediaWiki API error types
 //!
 //! The MediaWIki API is rather dynamic and has quite a few possible errors
@@ -22,18 +21,33 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 //! possible case as well as a conversion map between the API's error codes
 //! and Rust types.
 //!
+//! The `ApiError` type is serde-deserializable, and can be converted into
+//! a specific `Error` type using the API response code. Aside from serde,
+//! the library is fully library independent and should be usable by any
+//! MediaWiki library or framework.
+//!
+//! ## Features
 //! The `from` feature can be disabled to remove some dependencies that are
 //! used to implement the `From` trait. Each dependency can be individually
 //! toggled with a feature named `from-{dependency}`. Current features are:
+//! * `from-mwtitle`
 //! * `from-reqwest`
 //! * `from-tokio`
+//!
+//! ## Contributing
+//! `mwapi_errors` is a part of the [`mwbot-rs` project](https://www.mediawiki.org/wiki/Mwbot-rs).
+//! We're always looking for new contributors, please [reach out](https://www.mediawiki.org/wiki/Mwbot-rs#Contributing)
+//! if you're interested!
+
+#![cfg_attr(docs, feature(doc_cfg))]
+
 use serde::Deserialize;
 use serde_json::Value;
 use std::fmt::{Display, Formatter};
 use thiserror::Error as ThisError;
 
-/// Represents a raw MediaWiki API error, with a error code
-/// and error message (text).
+/// Represents a raw MediaWiki API error, with a error code and error message
+/// (text). This is also used for warnings since they use the same format.
 #[derive(Clone, Debug, Deserialize)]
 pub struct ApiError {
     /// Error code
@@ -57,11 +71,13 @@ pub enum Error {
     /* Request related errors */
     /// A HTTP error like a 4XX or 5XX status code
     #[cfg(feature = "from-reqwest")]
+    #[cfg_attr(docs, doc(cfg(feature = "from-reqwest")))]
     #[error("HTTP error: {0}")]
     HttpError(#[from] reqwest::Error),
     /// Invalid header value, likely if the provided OAuth2 token
     /// or User-agent are invalid
     #[cfg(feature = "from-reqwest")]
+    #[cfg_attr(docs, doc(cfg(feature = "from-reqwest")))]
     #[error("Invalid header value: {0}")]
     InvalidHeaderValue(#[from] reqwest::header::InvalidHeaderValue),
     /// Error when decoding the JSON response from the API
@@ -69,8 +85,14 @@ pub enum Error {
     JsonError(#[from] serde_json::Error),
     /// Error if unable to get request concurrency lock
     #[cfg(feature = "from-tokio")]
+    #[cfg_attr(docs, doc(cfg(feature = "from-tokio")))]
     #[error("Unable to get request lock: {0}")]
     LockError(#[from] tokio::sync::AcquireError),
+    #[cfg(feature = "from-mwtitle")]
+    #[cfg_attr(docs, doc(cfg(feature = "from-mwtitle")))]
+    #[error("Invalid title: {0}")]
+    InvalidTitle(#[from] mwtitle::Error),
+
     /// etag header is invalid/missing
     #[error("The etag for this request is missing or invalid")]
     InvalidEtag,
@@ -117,6 +139,8 @@ pub enum Error {
     WrongPassword,
 
     /* Page-related issues */
+    #[error("The specified title is not a valid page")]
+    InvalidPage,
     /// When {{nobots}} matches
     #[error("{{nobots}} prevents editing this page")]
     Nobots,
@@ -126,9 +150,11 @@ pub enum Error {
     /// Page is protected
     #[error("Page is protected")]
     ProtectedPage,
-    /// Edit conflict
-    #[error("Edit conflict on {0}")]
+    /// Edit conflict (string field is deprecated)
+    #[error("Edit conflict")]
     EditConflict(String),
+    #[error("Content too big: {0}")]
+    ContentTooBig(String),
     /// Tripped the spam filter (aka SpamBlacklist)
     #[error("{info}")]
     SpamFilter { info: String, matches: Vec<String> },
@@ -169,6 +195,7 @@ impl Error {
             Error::ProtectedPage
                 | Error::EditConflict(_)
                 | Error::SpamFilter { .. }
+                | Error::ContentTooBig(_)
         )
     }
 
@@ -243,6 +270,8 @@ impl From<ApiError> for Error {
                     None => Self::UnknownBlock(apierr.text),
                 }
             }
+            "contenttoobig" => Self::ContentTooBig(apierr.text),
+            "editconflict" => Self::EditConflict("".to_string()),
             "globalblocking-ipblocked"
             | "wikimedia-globalblocking-ipblocked" => {
                 Self::GloballyBlocked(apierr.text)
